@@ -424,7 +424,7 @@ function datatables(data, buttons, options, files) {
     if (options.dtopts.serverSide) {
         var url = data;
         // translate_datatable_group returns function which does the ajax data query, if doesn't exist, use url directly
-        $.extend(options.dtopts, { ajax: (translate_datatable_group && translate_datatable_group(url)) || url });
+        $.extend(options.dtopts, { ajax: (typeof translate_datatable_group === "function" && translate_datatable_group(url)) || url });
 
     // otherwise assume it is object containing the data to render
     } else {
