@@ -27,7 +27,6 @@ textreader - read text out of various file types
 '''
 
 # standard
-import pdb
 import argparse
 import csv
 
@@ -36,8 +35,7 @@ import csv
 # github
 
 # home grown
-import version
-from loutilities import *
+from . import version
 
 # valid file types
 VALIDFTYPES = ['xls','xlsx','docx','txt','csv']
@@ -49,6 +47,7 @@ TXTABSIZE = 8
 
 # exceptions for this module.  See __init__.py for package exceptions
 class headerError(Exception): pass
+class parameterError(Exception): pass
 
 ########################################################################
 class TextDictReader():
@@ -84,14 +83,14 @@ class TextDictReader():
     
         foundhdr = False
         delimited = self.file.getdelimited()
-        fields = self.fieldxform.keys()
+        fields = list(self.fieldxform.keys())
         MINMATCHES = len(self.reqdfields)
 
         # catch StopIteration, which means header wasn't found in the file
         try:
             # loop for each line until header found
             while True:
-                origline = self.file.next()
+                origline = next(self.file)
                 fieldsfound = 0
                 line = []
                 if not delimited:
@@ -114,7 +113,7 @@ class TextDictReader():
                         for linendx in range(len(line)):                        
                             # match over the end of the line is no match
                             # m is either a string or a list of strings
-                            if type(m) == str:
+                            if isinstance(m, str):
                                 m = [m]         # make single string into list
                             if linendx+len(m)>len(line):
                                 continue
@@ -142,11 +141,10 @@ class TextDictReader():
                         if f not in self.field:
                             fieldsnotfound.append(f)
                     if len(fieldsnotfound) != 0:
-                        raise headerError, 'could not find fields {} in header {}'.format(fieldsnotfound,origline)
+                        raise headerError('could not find fields {} in header {}'.format(fieldsnotfound, origline))
                         
                     # sort found fields by order found within the line
-                    foundfields_dec = [(self.field[f]['start'],self.field[f]) for f in self.field]
-                    foundfields_dec.sort()
+                    foundfields_dec = sorted([(self.field[f]['start'],self.field[f]) for f in self.field])
                     self.foundfields = [ff[1] for ff in foundfields_dec] # get rid of sorting decorator
                         
                     # here we have decided it is a header line
@@ -209,23 +207,23 @@ class TextDictReader():
                 
         # not good to come here
         except StopIteration:
-            raise headerError, 'header not found'
+            raise headerError('header not found')
         
     #----------------------------------------------------------------------
-    def next(self):
+    def __next__(self):
     #----------------------------------------------------------------------
         '''
         return dict with generic headers and associated data from file
         '''
         
         # get next raw line from the file
-        rawline = self.file.next()
+        rawline = next(self.file)
         
         # pick columns which are associated with generic headers
         filteredline = [rawline[i] for i in range(len(rawline)) if i in self.fieldcols]
         
         # create dict association, similar to csv.DictReader
-        result = dict(zip(self.fieldhdrs,filteredline))
+        result = dict(list(zip(self.fieldhdrs,filteredline)))
                    
         # and return result
         return result
@@ -251,28 +249,38 @@ class TextReader():
         """
 
         # if true filename, type of filename is string
-        if type(filename) in [str,unicode]:
+        if isinstance(filename, str):
             self.ftype = filename.split('.')[-1].lower() # get extension
             self.intype = 'file'
             if self.ftype not in VALIDFTYPES:
-                raise parameterError, 'Invalid filename {}: must have extension in {}'.format(filename,VALIDFTYPES)
+                raise parameterError('Invalid filename {}: must have extension in {}'.format(filename, VALIDFTYPES))
         # otherwise assume 'filename' is list-like
         else:
             self.ftype = filetype.lower()
             self.intype = 'list'
             if self.ftype not in VALIDLTYPES:
-                raise parameterError, 'Invalid list: must use filetype in {}'.format(VALIDLTYPES)
+                raise parameterError('Invalid list: must use filetype in {}'.format(VALIDLTYPES))
         
         # handle excel files
-        if self.ftype in ['xls','xlsx']:
-            import xlrd
-            self.workbook = xlrd.open_workbook(filename)
+        if self.ftype in ['xls']:
+            from xlrd import open_workbook
+            self.workbook = open_workbook(filename)
             self.sheet = self.workbook.sheet_by_index(0)    # only first sheet is considered
             self.currrow = 0
             self.nrows = self.sheet.nrows
             self.delimited = True               # rows are already broken into columns
             self.workbook.release_resources()   # sheet is already loaded so we can save memory
             
+        # handle excel files
+        elif self.ftype in ['xlsx']:
+            from openpyxl import load_workbook
+            self.workbook = load_workbook(filename)
+            self.sheet = self.workbook[self.workbook.sheetnames[0]]    # only first sheet is considered
+            self.currrow = 0
+            self.rows = list(self.sheet.rows)
+            self.nrows = len(list(self.sheet.rows))
+            self.delimited = True               # rows are already broken into columns
+
         # handle word files
         elif self.ftype in ['docx']:
             import docx
@@ -291,7 +299,7 @@ class TextReader():
         # handle txt files
         elif self.ftype in ['csv']:
             if self.intype == 'file':
-                self._CSV = open(filename,'rb')
+                self._CSV = open(filename,'r')
             else:
                 self._CSV = iter(filename)
             self.CSV = csv.reader(self._CSV)
@@ -301,7 +309,7 @@ class TextReader():
         self.opened = True
         
     #----------------------------------------------------------------------
-    def next(self):
+    def __next__(self):
     #----------------------------------------------------------------------
         """
         read next line from TextReader file
@@ -312,10 +320,10 @@ class TextReader():
         
         # check if it is ok to read
         if not self.opened:
-            raise ValueError, 'I/O operation on a closed file'
+            raise ValueError('I/O operation on a closed file')
         
         # handle excel files
-        if self.ftype in ['xls','xlsx']:
+        if self.ftype in ['xls']:
             if self.currrow >= self.nrows:
                 raise StopIteration
             
@@ -323,6 +331,15 @@ class TextReader():
             self.currrow += 1
             return row
         
+        # handle excel files
+        if self.ftype in ['xlsx']:
+            if self.currrow >= self.nrows:
+                raise StopIteration
+            
+            row = [c.value for c in self.rows[self.currrow]]
+            self.currrow += 1
+            return row
+        
         # handle word files
         elif self.ftype in ['docx']:
             line = next(self.lines)
@@ -345,7 +362,7 @@ class TextReader():
 
         # handle txt files
         elif self.ftype in ['csv']:
-            row = self.CSV.next()
+            row = next(self.CSV)
             return row
 
     #----------------------------------------------------------------------
@@ -400,13 +417,13 @@ class TextReader():
         
         # do some validation
         if self.delimited:
-            raise parameterError, 'cannot set delimiters for file which is already delimited'
-        if type(delimiters) != list:
-            raise parameterError, 'delimiters must be a list of increasing integers'
+            raise parameterError('cannot set delimiters for file which is already delimited')
+        if not isinstance(delimiters, list):
+            raise parameterError('delimiters must be a list of increasing integers')
         lastdelimiter = -1
         for delimiter in delimiters:
-            if type(delimiter) != int or delimiter <= lastdelimiter:
-                raise parameterError, 'delimiters must be a list of increasing integers'
+            if not isinstance(delimiter, int) or delimiter <= lastdelimiter:
+                raise parameterError('delimiters must be a list of increasing integers')
             lastdelimiter = delimiter
             
         self.delimiters = delimiters
@@ -423,7 +440,7 @@ class TextReader():
         """
         
         if not self.delimiters:
-            raise parameterError, 'cannot split string if delimiters not set'
+            raise parameterError('cannot split string if delimiters not set')
         
         rval = []
         for i in range(len(self.delimiters)):
@@ -447,7 +464,7 @@ def main():
     # open file, print some lines, then close
     ff = TextReader(filename)
     for i in range(6):
-        print(ff.next())
+        print(next(ff))
     ff.close()
 
 # ###############################################################################
